@@ -23,47 +23,76 @@
 #include <cupt/cache/version.hpp>
 
 namespace cupt {
+namespace internal {
+
+struct VersionParseParameters;
+
+template< typename VersionType >
+class CUPT_API BasePackageIterator: public std::iterator< std::bidirectional_iterator_tag, const VersionType* >
+{
+	friend class cache::Package;
+	friend class cache::BinaryPackage;
+	friend class cache::SourcePackage;
+
+	typedef vector< unique_ptr< cache::Version > >::const_iterator UnderlyingIterator;
+
+	UnderlyingIterator __ui;
+
+	BasePackageIterator(UnderlyingIterator);
+ public:
+	typedef BasePackageIterator Self;
+
+	Self& operator++();
+	const VersionType* operator*() const;
+	bool operator==(const Self&) const;
+	bool operator!=(const Self&) const;
+};
+
+}
+
 namespace cache {
 
 /// a container for all versions of the same package name
 class CUPT_API Package
 {
-	mutable vector< Version::InitializationParameters > __unparsed_versions;
-	mutable vector< shared_ptr< Version > >* __parsed_versions;
+	vector< unique_ptr< Version > > __parsed_versions;
 
-	CUPT_LOCAL void __merge_version(shared_ptr< Version >&&, vector< shared_ptr< Version > >& result) const;
+	CUPT_LOCAL void __merge_version(unique_ptr< Version >&&);
 
 	Package(const Package&);
 	Package& operator=(const Package&);
  protected:
 	/// @cond
-	shared_ptr< const string > _binary_architecture;
+	const string* _binary_architecture;
 
-	CUPT_LOCAL vector< shared_ptr< Version > > _get_versions() const;
-	CUPT_LOCAL virtual shared_ptr< Version > _parse_version(const Version::InitializationParameters&) const = 0;
-	CUPT_LOCAL virtual bool _is_architecture_appropriate(const shared_ptr< const Version >&) const = 0;
+	CUPT_LOCAL const vector< unique_ptr< Version > >& _get_versions() const;
+	CUPT_LOCAL virtual unique_ptr< Version > _parse_version(const internal::VersionParseParameters&) const = 0;
+	CUPT_LOCAL virtual bool _is_architecture_appropriate(const Version*) const = 0;
 	/// @endcond
  public:
 	/// constructor
 	/**
 	 * @param binaryArchitecture binary architecture of the system
 	 */
-	Package(const shared_ptr< const string >& binaryArchitecture);
+	Package(const string* binaryArchitecture);
 	/// destructor
 	virtual ~Package();
-	/// adds new element (version initialization parameters) to the container
-	void addEntry(const Version::InitializationParameters&);
+	/// @cond
+	CUPT_LOCAL void addEntry(const internal::VersionParseParameters&);
+	/// @endcond
+
 	/// gets list of versions
-	vector< shared_ptr< const Version > > getVersions() const;
+	vector< const Version* > getVersions() const;
 	/// gets version with a certain Version::versionString
 	/**
 	 * @return version if found, empty pointer if not found
 	 * @param versionString version string
 	 */
-	shared_ptr< const Version > getSpecificVersion(const string& versionString) const;
+	const Version* getSpecificVersion(const string& versionString) const;
 
-	/// memoize parsed versions
-	static bool memoize;
+	typedef internal::BasePackageIterator< Version > iterator;
+	iterator begin() const;
+	iterator end() const;
 };
 
 }
