@@ -20,27 +20,33 @@
 
 #include <cupt/fwd.hpp>
 #include <cupt/cache.hpp>
+#include <cupt/hashsums.hpp>
 
 namespace cupt {
 namespace internal {
 namespace cachefiles {
 
 typedef Cache::IndexEntry IndexEntry;
-typedef Cache::IndexDownloadRecord FileDownloadRecord;
+struct FileDownloadRecord
+{
+	string uri;
+	uint32_t size;
+	HashSums hashSums;
+};
 
 string getPathOfIndexList(const Config&, const IndexEntry&);
 string getPathOfReleaseList(const Config&, const IndexEntry&);
+string getPathOfInReleaseList(const Config&, const IndexEntry&);
+string getPathOfMasterReleaseLikeList(const Config&, const IndexEntry&);
 string getPathOfExtendedStates(const Config&);
 
 string getDownloadUriOfReleaseList(const IndexEntry&);
+string getDownloadUriOfInReleaseList(const IndexEntry&);
 vector< FileDownloadRecord > getDownloadInfoOfIndexList(
 		const Config&, const IndexEntry&);
 
 vector< pair< string, string > > getPathsOfLocalizedDescriptions(
 		const Config&, const IndexEntry& entry);
-// TODO/API break/: deprecated, delete it
-vector< Cache::LocalizationDownloadRecord > getDownloadInfoOfLocalizedDescriptions(
-		const Config&, const IndexEntry&);
 
 vector< FileDownloadRecord > getDownloadInfoOfLocalizationIndex(
 		const Config&, const IndexEntry&);
