@@ -34,20 +34,18 @@ namespace cupt {
 namespace internal {
 namespace dependencygraph {
 
+enum VertexTypePriority { Zero, SuggestsRelationExpression, RecommendsRelationExpression, WishRequest, TryRequest, StrongRelationExpression, MustRequest };
+
 using cache::RelationExpression;
 using std::make_pair;
 
-InitialPackageEntry::InitialPackageEntry()
-	: version(NULL), modified(false)
-{}
-
 BasicVertex::~BasicVertex()
 {}
 
 size_t BasicVertex::getTypePriority() const
 {
 	fatal2i("getting priority of '%s'", toString());
-	return 0; // unreachable
+	return VertexTypePriority::Zero; // unreachable
 }
 
 uint32_t BasicVertex::__next_id = 0;
@@ -68,7 +66,7 @@ shared_ptr< const Reason > BasicVertex::getReason(const BasicVertex&) const
 	return shared_ptr< const Reason >(); // unreachable
 }
 
-const forward_list< const Element* >* BasicVertex::getRelatedElements() const
+const forward_list<Element>* BasicVertex::getRelatedElements() const
 {
 	fatal2i("getting related elements of '%s'", toString());
 	return NULL; // unreachable
@@ -91,7 +89,23 @@ bool BasicVertex::asAuto() const
 	return true; // unreacahble
 }
 
-VersionVertex::VersionVertex(const map< string, forward_list< const Element* > >::iterator& it)
+Element BasicVertex::getFamilyKey() const
+{
+	return this;
+}
+
+
+struct ExtendedBasicVertex: public BasicVertex
+{
+	virtual const string& getSpecificPackageName() const
+	{
+		fatal2i("getting getSpecificPackageName of '%s'", toString());
+		__builtin_unreachable();
+	}
+};
+
+
+VersionVertex::VersionVertex(const FamilyMap::iterator& it)
 	: __related_element_ptrs_it(it)
 {}
 
@@ -101,9 +115,9 @@ string VersionVertex::toString() const
 			(version ? version->versionString : "<not installed>");
 }
 
-const forward_list< const Element* >* VersionVertex::getRelatedElements() const
+const forward_list<Element>* VersionVertex::getRelatedElements() const
 {
-	return &__related_element_ptrs_it->second;
+	return &__related_element_ptrs_it->second.first;
 }
 
 const string& VersionVertex::getPackageName() const
@@ -124,9 +138,14 @@ string VersionVertex::toLocalizedString() const
 	}
 }
 
+Element VersionVertex::getFamilyKey() const
+{
+	return __related_element_ptrs_it->second.second;
+}
+
 typedef BinaryVersion::RelationTypes::Type RelationType;
 
-struct RelationExpressionVertex: public BasicVertex
+struct RelationExpressionVertex: public ExtendedBasicVertex
 {
 	RelationType dependencyType;
 	const RelationExpression* relationExpressionPtr;
@@ -154,15 +173,15 @@ size_t RelationExpressionVertex::getTypePriority() const
 	{
 		case BinaryVersion::RelationTypes::PreDepends:
 		case BinaryVersion::RelationTypes::Depends:
-			return 3;
+			return VertexTypePriority::StrongRelationExpression;
 		case BinaryVersion::RelationTypes::Recommends:
-			return 2;
+			return VertexTypePriority::RecommendsRelationExpression;
 		case BinaryVersion::RelationTypes::Suggests:
-			return 1;
+			return VertexTypePriority::SuggestsRelationExpression;
 		default:
 			fatal2i("unsupported dependency type '%d'", int(dependencyType));
 	}
-	return 0; // unreacahble
+	return VertexTypePriority::Zero; // unreacahble
 }
 
 bool RelationExpressionVertex::isAnti() const
@@ -207,7 +226,7 @@ struct AntiRelationExpressionVertex: public RelationExpressionVertex
 	}
 	size_t getTypePriority() const
 	{
-		return 3;
+		return VertexTypePriority::StrongRelationExpression;
 	}
 	bool isAnti() const
 	{
@@ -217,6 +236,10 @@ struct AntiRelationExpressionVertex: public RelationExpressionVertex
 	{
 		return Unsatisfied::None;
 	}
+	const string& getSpecificPackageName() const
+	{
+		return specificPackageName;
+	}
 };
 
 struct SynchronizeVertex: public BasicVertex
@@ -243,7 +266,7 @@ string SynchronizeVertex::toString() const
 
 size_t SynchronizeVertex::getTypePriority() const
 {
-	return isHard ? 3 : 2;
+	return isHard ? VertexTypePriority::StrongRelationExpression : VertexTypePriority::WishRequest;
 }
 
 shared_ptr< const Reason > SynchronizeVertex::getReason(const BasicVertex& parent) const
@@ -269,10 +292,10 @@ Unsatisfied::Type SynchronizeVertex::getUnsatisfiedType() const
 
 struct UnsatisfiedVertex: public BasicVertex
 {
-	const Element* parent;
+	Element parent;
 
 	string toString() const;
-	const forward_list< const Element* >* getRelatedElements() const;
+	const forward_list<Element>* getRelatedElements() const;
 	Unsatisfied::Type getUnsatisfiedType() const;
 };
 
@@ -282,7 +305,7 @@ string UnsatisfiedVertex::toString() const
 	return u + parent->toString();
 }
 
-const forward_list< const Element* >* UnsatisfiedVertex::getRelatedElements() const
+const forward_list<Element>* UnsatisfiedVertex::getRelatedElements() const
 {
 	return NULL;
 }
@@ -322,20 +345,39 @@ class AnnotatedUserReason: public system::Resolver::UserReason
 	}
 };
 
-struct UserRelationExpressionVertex: public BasicVertex
+static inline uint8_t getTypePriorityForUserRequest(const UserRelationExpression& ure)
+{
+	if (ure.importance == RequestImportance::Must)
+	{
+		return VertexTypePriority::MustRequest;
+	}
+	else if (ure.importance == RequestImportance::Try)
+	{
+		return VertexTypePriority::TryRequest;
+	}
+	else
+	{
+		return VertexTypePriority::WishRequest;
+	}
+}
+
+struct UserRelationExpressionVertex: public ExtendedBasicVertex
 {
 	bool invert;
 	bool asAutoFlag;
+	uint16_t typePriority;
 	string annotation;
+	string specificPackageName;
 
 	UserRelationExpressionVertex(const UserRelationExpression& ure)
 		: invert(ure.invert)
 		, asAutoFlag(ure.asAuto)
+		, typePriority(getTypePriorityForUserRequest(ure))
 		, annotation(ure.annotation)
 	{}
 	size_t getTypePriority() const
 	{
-		return 4;
+		return typePriority;
 	}
 	bool isAnti() const
 	{
@@ -353,6 +395,10 @@ struct UserRelationExpressionVertex: public BasicVertex
 	{
 		return "custom: " + annotation;
 	}
+	const string& getSpecificPackageName() const
+	{
+		return specificPackageName;
+	}
 };
 
 bool __is_version_array_intersects_with_packages(
@@ -490,10 +536,9 @@ DependencyGraph::DependencyGraph(const Config& config, const Cache& cache)
 
 DependencyGraph::~DependencyGraph()
 {
-	const set< const Element* >& vertices = this->getVertices();
-	FORIT(elementIt, vertices)
+	for (auto element: this->getVertices())
 	{
-		delete *elementIt;
+		delete element;
 	}
 }
 
@@ -564,14 +609,15 @@ class DependencyGraph::FillHelper
 	int __synchronize_level;
 	vector< DependencyEntry > __dependency_groups;
 
-	map< string, forward_list< const Element* > > __package_name_to_vertex_ptrs;
+	typedef pair< forward_list<Element>, Element > RelatedVertexPtrs;
+	map< string, RelatedVertexPtrs > __package_name_to_vertex_ptrs;
 	unordered_map< string, const VersionVertex* > __version_to_vertex_ptr;
-	unordered_map< string, const Element* > __relation_expression_to_vertex_ptr;
-	unordered_map< string, map< string, const Element* > > __meta_anti_relation_expression_vertices;
-	unordered_map< string, list< pair< string, const Element* > > > __meta_synchronize_map;
-	const Element* p_dummyElementPtr;
+	unordered_map< string, Element > __relation_expression_to_vertex_ptr;
+	unordered_map< string, list<const ExtendedBasicVertex*> > __meta_anti_relation_expression_vertices;
+	unordered_map< string, list< pair< string, Element > > > __meta_synchronize_map;
+	Element p_dummyElementPtr;
 
-	set< const Element* > __unfolded_elements;
+	set<Element> __unfolded_elements;
 
 	bool __can_package_be_removed(const string& packageName) const
 	{
@@ -603,7 +649,7 @@ class DependencyGraph::FillHelper
 
 			if (version)
 			{
-				for (const BasicVertex* bv: __package_name_to_vertex_ptrs[packageName])
+				for (const BasicVertex* bv: __package_name_to_vertex_ptrs[packageName].first)
 				{
 					auto existingVersion = (static_cast< const VersionVertex* >(bv))->version;
 					if (!existingVersion) continue;
@@ -624,11 +670,15 @@ class DependencyGraph::FillHelper
 		auto makeVertex = [this, &packageName, &version]() -> const VersionVertex*
 		{
 			auto relatedVertexPtrsIt = __package_name_to_vertex_ptrs.insert(
-					{ packageName, {} }).first;
+					{ packageName, RelatedVertexPtrs() }).first;
 			auto vertexPtr(new VersionVertex(relatedVertexPtrsIt));
 			vertexPtr->version = version;
 			__dependency_graph.addVertex(vertexPtr);
-			relatedVertexPtrsIt->second.push_front(vertexPtr);
+			relatedVertexPtrsIt->second.first.push_front(vertexPtr);
+			if (!relatedVertexPtrsIt->second.second)
+			{
+				relatedVertexPtrsIt->second.second = vertexPtr; // family vertex
+			}
 			return vertexPtr;
 		};
 
@@ -646,50 +696,66 @@ class DependencyGraph::FillHelper
 		return *elementPtrPtr;
 	}
 
-	const VersionElement* getVertexPtr(const BinaryVersion* version, bool overrideChecks = false)
+	VersionElement getVertexPtr(const BinaryVersion* version, bool overrideChecks = false)
 	{
 		return getVertexPtr(version->packageName, version, overrideChecks);
 	}
 
  private:
-	void addEdgeCustom(const Element* fromVertexPtr, const Element* toVertexPtr)
+	void addEdgeCustom(Element fromVertexPtr, Element toVertexPtr)
 	{
-		__dependency_graph.addEdgeFromPointers(fromVertexPtr, toVertexPtr);
+		__dependency_graph.addEdge(fromVertexPtr, toVertexPtr);
 	}
 
-	const Element* getVertexPtrForRelationExpression(const RelationExpression* relationExpressionPtr,
+	Element getVertexPtrForRelationExpression(const RelationExpression* relationExpressionPtr,
 			const RelationType& dependencyType, bool* isNew)
 	{
 		auto hashKey = relationExpressionPtr->getHashString() + char('0' + dependencyType);
-		const Element*& elementPtr = __relation_expression_to_vertex_ptr.insert(
-				make_pair(std::move(hashKey), (const Element*)NULL)).first->second;
-		*isNew = !elementPtr;
-		if (!elementPtr)
+		Element& element = __relation_expression_to_vertex_ptr.insert(
+				make_pair(std::move(hashKey), nullptr)).first->second;
+		*isNew = !element;
+		if (!element)
 		{
 			auto vertex(new RelationExpressionVertex);
 			vertex->dependencyType = dependencyType;
 			vertex->relationExpressionPtr = relationExpressionPtr;
-			elementPtr = __dependency_graph.addVertex(vertex);
+			element = __dependency_graph.addVertex(vertex);
 		}
-		return elementPtr;
+		return element;
 	}
 
  public:
-	const Element* getVertexPtrForEmptyPackage(const string& packageName)
+	Element getVertexPtrForEmptyPackage(const string& packageName, bool /* overrideChecks */ = false)
 	{
-		return getVertexPtr(packageName, nullptr);
+		return getVertexPtr(packageName, nullptr, false);
 	}
 
  private:
+	const ExtendedBasicVertex*& getOrCreateListSubElement(
+			list<const ExtendedBasicVertex*>& l, const string& packageName)
+	{
+		for (auto& e: l)
+		{
+			if (e->getSpecificPackageName() == packageName)
+			{
+				return e;
+			}
+		}
+
+		l.push_back(nullptr);
+		return l.back();
+	}
+
 	void buildEdgesForAntiRelationExpression(
-			map< string, const Element* >* packageNameToSubElements,
+			list<const ExtendedBasicVertex*>* packageNameToSubElements,
 			const vector< const BinaryVersion* > satisfyingVersions,
-			const std::function< const Element* (const string&) >& createVertex)
+			const std::function< const ExtendedBasicVertex* (const string&) >& createVertex,
+			bool overrideChecks = false)
 	{
 		for (auto satisfyingVersion: satisfyingVersions)
 		{
 			const string& packageName = satisfyingVersion->packageName;
-			auto& subElement = (*packageNameToSubElements)[packageName];
+			auto& subElement = getOrCreateListSubElement(*packageNameToSubElements, packageName);
 			if (subElement) continue;
 
 			subElement = createVertex(packageName);
@@ -697,32 +763,33 @@ class DependencyGraph::FillHelper
 			auto package = __dependency_graph.__cache.getBinaryPackage(packageName);
 			if (!package) fatal2i("the binary package '%s' doesn't exist", packageName);
 
-			for (auto packageVersion: *package)
+			for (auto pinnedVersion: __dependency_graph.__cache.getSortedPinnedVersions(package))
 			{
+				auto packageVersion = static_cast<const BinaryVersion*>(pinnedVersion.version);
 				if (std::find(satisfyingVersions.begin(), satisfyingVersions.end(),
 							packageVersion) == satisfyingVersions.end())
 				{
-					if (auto queuedVersionPtr = getVertexPtr(packageVersion))
+					if (auto queuedVersionPtr = getVertexPtr(packageVersion, overrideChecks))
 					{
 						addEdgeCustom(subElement, queuedVersionPtr);
 					}
 				}
 			}
 
-			if (auto emptyPackageElementPtr = getVertexPtrForEmptyPackage(packageName))
+			if (auto emptyPackageElementPtr = getVertexPtrForEmptyPackage(packageName, overrideChecks))
 			{
 				addEdgeCustom(subElement, emptyPackageElementPtr);
 			}
 		}
 	}
 	void processAntiRelation(const string& packageName,
-			const Element* vertexPtr, const RelationExpression& relationExpression,
+			Element vertexPtr, const RelationExpression& relationExpression,
 			BinaryVersion::RelationTypes::Type dependencyType)
 	{
 		auto hashKey = relationExpression.getHashString() + char('0' + dependencyType);
-		static const map< string, const Element* > emptyMap;
+		static const list<const ExtendedBasicVertex*> emptyList;
 		auto insertResult = __meta_anti_relation_expression_vertices.insert(
-				make_pair(std::move(hashKey), emptyMap));
+				make_pair(std::move(hashKey), emptyList));
 		bool isNewRelationExpressionVertex = insertResult.second;
 		auto& packageNameToSubElements = insertResult.first->second;
 
@@ -734,23 +801,24 @@ class DependencyGraph::FillHelper
 				subVertex->dependencyType = dependencyType;
 				subVertex->relationExpressionPtr = &relationExpression;
 				subVertex->specificPackageName = packageName;
-				return __dependency_graph.addVertex(subVertex);
+				__dependency_graph.addVertex(subVertex);
+				return subVertex;
 			};
 			auto satisfyingVersions = __dependency_graph.__cache.getSatisfyingVersions(relationExpression);
 			buildEdgesForAntiRelationExpression(&packageNameToSubElements, satisfyingVersions, createVertex);
 		}
 		for (const auto& it: packageNameToSubElements)
 		{
-			if (it.first == packageName)
+			if (it->getSpecificPackageName() == packageName)
 			{
 				continue; // doesn't conflict with itself
 			}
-			addEdgeCustom(vertexPtr, it.second);
+			addEdgeCustom(vertexPtr, it);
 		}
 	}
 
 	void processForwardRelation(const BinaryVersion* version,
-			const Element* vertexPtr, const RelationExpression& relationExpression,
+			Element vertexPtr, const RelationExpression& relationExpression,
 			BinaryVersion::RelationTypes::Type dependencyType)
 	{
 		vector< const BinaryVersion* > satisfyingVersions;
@@ -807,13 +875,13 @@ class DependencyGraph::FillHelper
 		}
 	}
 
-	void processSynchronizations(const BinaryVersion*& version, const Element* vertexPtr)
+	void processSynchronizations(const BinaryVersion*& version, Element vertexPtr)
 	{
 		auto hashKey = version->sourcePackageName + ' ' + version->sourceVersionString;
-		static const list< pair< string, const Element* > > emptyList;
+		static const list< pair< string, Element > > emptyList;
 		auto insertResult = __meta_synchronize_map.insert(make_pair(hashKey, emptyList));
 		bool isNewMetaVertex = insertResult.second;
-		list< pair< string, const Element* > >& subElementPtrs = insertResult.first->second;
+		auto& subElementPtrs = insertResult.first->second;
 
 		if (isNewMetaVertex)
 		{
@@ -860,7 +928,7 @@ class DependencyGraph::FillHelper
 		}
 	}
 
-	const Element* createCustomUnsatisfiedElement(const Element* parent, const RequestImportance& importance)
+	Element createCustomUnsatisfiedElement(Element parent, const RequestImportance& importance)
 	{
 		auto vertex = new CustomUnsatisfiedVertex(importance);
 		vertex->parent = parent;
@@ -868,20 +936,20 @@ class DependencyGraph::FillHelper
 	}
 
  public:
-	void unfoldElement(const Element* elementPtr)
+	void unfoldElement(Element element)
 	{
-		if (!__unfolded_elements.insert(elementPtr).second)
+		if (!__unfolded_elements.insert(element).second)
 		{
 			return; // processed already
 		}
-		auto versionElementPtr = dynamic_cast< const VersionElement* >(elementPtr);
-		if (!versionElementPtr)
+		auto versionElement = dynamic_cast<VersionElement>(element);
+		if (!versionElement)
 		{
 			return; // nothing to process
 		}
 
 		// persistent one
-		auto version = versionElementPtr->version;
+		auto version = versionElement->version;
 		if (!version)
 		{
 			return;
@@ -896,33 +964,34 @@ class DependencyGraph::FillHelper
 			{
 				if (isDependencyAnti)
 				{
-					processAntiRelation(version->packageName, elementPtr, relationExpression, dependencyType);
+					processAntiRelation(version->packageName, element, relationExpression, dependencyType);
 				}
 				else
 				{
-					processForwardRelation(version, elementPtr, relationExpression, dependencyType);
+					processForwardRelation(version, element, relationExpression, dependencyType);
 				}
 			}
 		}
 
 		if (__synchronize_level && !version->isInstalled())
 		{
-			processSynchronizations(version, elementPtr);
+			processSynchronizations(version, element);
 		}
 	}
 
-	const Element* getDummyElementPtr() const
+	Element getDummyElementPtr() const
 	{
 		return p_dummyElementPtr;
 	}
 
 	void addUserRelationExpression(const UserRelationExpression& ure)
 	{
-		const Element* unsatisfiedElement = nullptr;
+		Element unsatisfiedElement = nullptr;
 
-		auto createVertex = [&](const string&) -> const Element*
+		auto createVertex = [&](const string& packageName) -> const ExtendedBasicVertex*
 		{
 			auto vertex = new UserRelationExpressionVertex(ure);
+			vertex->specificPackageName = packageName;
 			__dependency_graph.addVertex(vertex);
 			addEdgeCustom(p_dummyElementPtr, vertex);
 			if (ure.importance != RequestImportance::Must)
@@ -945,60 +1014,58 @@ class DependencyGraph::FillHelper
 		}
 		else
 		{
-			map< string, const Element* > subElements;
-			buildEdgesForAntiRelationExpression(&subElements, satisfyingVersions, createVertex);
+			list<const ExtendedBasicVertex*> subElements;
+			buildEdgesForAntiRelationExpression(&subElements, satisfyingVersions, createVertex, true);
 		}
 	}
 };
 
-const shared_ptr< const PackageEntry >& getSharedPackageEntry(bool sticked)
-{
-	static const auto stickedOne = std::make_shared< const PackageEntry >(true);
-	static const auto notStickedOne = std::make_shared< const PackageEntry >(false);
-	return sticked ? stickedOne : notStickedOne;
-}
-
-vector< pair< const dg::Element*, shared_ptr< const PackageEntry > > > DependencyGraph::fill(
-		const map< string, const BinaryVersion* >& oldPackages,
-		const map< string, InitialPackageEntry >& initialPackages)
+vector< pair< dg::Element, shared_ptr< const PackageEntry > > > DependencyGraph::fill(
+		const map< string, const BinaryVersion* >& oldPackages)
 {
 	__fill_helper.reset(new DependencyGraph::FillHelper(*this, oldPackages));
 
 	{ // getting elements from initial packages
-		FORIT(it, initialPackages)
+		for (const auto& item: oldPackages)
 		{
-			const InitialPackageEntry& initialPackageEntry = it->second;
-			const auto& initialVersion = initialPackageEntry.version;
+			const auto& oldVersion = item.second;
 
-			if (initialVersion)
+			__fill_helper->getVertexPtr(oldVersion);
+
+			const string& packageName = oldVersion->packageName;
+			auto package = __cache.getBinaryPackage(packageName);
+			for (auto version: *package)
 			{
-				__fill_helper->getVertexPtr(initialVersion);
-
-				const string& packageName = it->first;
-				auto package = __cache.getBinaryPackage(packageName);
-				for (auto version: *package)
-				{
-					__fill_helper->getVertexPtr(version);
-				}
-
-				__fill_helper->getVertexPtrForEmptyPackage(packageName); // also, empty one
+				__fill_helper->getVertexPtr(version);
 			}
+
+			__fill_helper->getVertexPtrForEmptyPackage(packageName); // also, empty one
 		}
 	}
 
-	return p_generateSolutionElements(initialPackages);
+	return p_generateSolutionElements(oldPackages);
 }
 
-vector< pair< const dg::Element*, shared_ptr< const PackageEntry > > > DependencyGraph::p_generateSolutionElements(
-		const map< string, InitialPackageEntry >& initialPackages)
+vector< pair< dg::Element, shared_ptr< const PackageEntry > > > DependencyGraph::p_generateSolutionElements(
+		const map< string, const BinaryVersion* >& oldPackages)
 {
-	vector< pair< const Element*, shared_ptr< const PackageEntry > > > result;
-	for (const auto& it: initialPackages)
+	vector< pair< Element, shared_ptr< const PackageEntry > > > result;
+
+	auto generate = [&result](Element element, bool sticked)
 	{
-		auto elementPtr = __fill_helper->getVertexPtr(it.first, it.second.version);
-		result.push_back({ elementPtr, getSharedPackageEntry(false) });
+		auto packageEntry = std::make_shared<PackageEntry>();
+		packageEntry->element = element;
+		packageEntry->sticked = sticked;
+
+		result.emplace_back(element->getFamilyKey(), packageEntry);
+	};
+
+	for (const auto& it: oldPackages)
+	{
+		generate(__fill_helper->getVertexPtr(it.second), false);
 	}
-	result.emplace_back(__fill_helper->getDummyElementPtr(), getSharedPackageEntry(true));
+	generate(__fill_helper->getDummyElementPtr(), true);
+
 	return result;
 }
 
@@ -1007,14 +1074,14 @@ void DependencyGraph::addUserRelationExpression(const UserRelationExpression& ur
 	__fill_helper->addUserRelationExpression(ure);
 }
 
-void DependencyGraph::unfoldElement(const Element* elementPtr)
+void DependencyGraph::unfoldElement(Element element)
 {
-	__fill_helper->unfoldElement(elementPtr);
+	__fill_helper->unfoldElement(element);
 }
 
-const Element* DependencyGraph::getCorrespondingEmptyElement(const Element* elementPtr)
+Element DependencyGraph::getCorrespondingEmptyElement(Element element)
 {
-	auto versionVertex = dynamic_cast< const VersionVertex* >(elementPtr);
+	auto versionVertex = dynamic_cast<VersionElement>(element);
 	if (!versionVertex)
 	{
 		fatal2i("getting corresponding empty element for non-version vertex");
