@@ -15,12 +15,16 @@
 *   Free Software Foundation, Inc.,                                       *
 *   51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA               *
 **************************************************************************/
+#include <unistd.h>
+
+#include <thread>
+
 #include <cupt/config.hpp>
-#include <cupt/file.hpp>
 #include <cupt/versionstring.hpp>
 
 #include <internal/lock.hpp>
 #include <internal/common.hpp>
+#include <internal/pipe.hpp>
 
 #include <internal/worker/base.hpp>
 
@@ -66,54 +70,34 @@ string WorkerBase::_get_archive_basename(const BinaryVersion* version)
 }
 
 void WorkerBase::_run_external_command(Logger::Subsystem subsystem,
-		const string& command, const string& commandInput, const string& errorId)
+		const string& command, const CommandInput& commandInput, const string& errorId)
 {
 	const Logger::Level level = 3;
 	_logger->log(subsystem, level, format2("running: %s", command));
 
 	if (_config->getBool("cupt::worker::simulate"))
 	{
-		if (commandInput.empty())
+		if (commandInput.buffer.empty())
 		{
 			simulate2("running command '%s'", command);
 		}
 		else
 		{
 			simulate2("running command '%s' with the input\n-8<-\n%s\n->8-",
-					command, commandInput);
+					command, commandInput.buffer);
 		}
 	}
 	else
 	{
 		const string& id = (errorId.empty() ? command : errorId);
 
-		if (commandInput.empty())
+		if (commandInput.buffer.empty())
 		{
-			// invoking command
-			auto result = ::system(command.c_str());
-			if (result == -1)
-			{
-				_logger->loggedFatal2(subsystem, level,
-						format2e, "unable to launch the command '%s'", command);
-			}
-			else if (result)
-			{
-				_logger->loggedFatal2(subsystem, level,
-						format2, "the command '%s' failed: %s", command, getWaitStatusDescription(result));
-			}
+			p_invokeShellCommand(subsystem, level, command);
 		}
 		else try
 		{
-			// invoking command
-			string errorString;
-			File pipeFile(command, "pw", errorString);
-			if (!errorString.empty())
-			{
-				_logger->loggedFatal2(subsystem, level,
-						format2, "unable to open the pipe '%s': %s", command, errorString);
-			}
-
-			pipeFile.put(commandInput);
+			p_runCommandWithInput(subsystem, level, command, commandInput);
 		}
 		catch (...)
 		{
@@ -122,6 +106,73 @@ void WorkerBase::_run_external_command(Logger::Subsystem subsystem,
 	}
 }
 
+void WorkerBase::p_invokeShellCommand(Logger::Subsystem subsystem, Logger::Level level, const string& command)
+{
+	auto result = ::system(command.c_str());
+	if (result == -1)
+	{
+		_logger->loggedFatal2(subsystem, level,
+				format2e, "unable to launch the command '%s'", command);
+	}
+	else if (result)
+	{
+		_logger->loggedFatal2(subsystem, level,
+				format2, "the command '%s' failed: %s", command, getWaitStatusDescription(result));
+	}
+}
+
+static void writeBufferToFd(Pipe& pipe, const string& buffer, const string& pipeName)
+{
+	int fd = pipe.getWriterFd();
+	const char* bufferCursor = buffer.data();
+	const char* bufferEnd = bufferCursor + buffer.size();
+
+	while (bufferCursor < bufferEnd)
+	{
+		auto writeResult = ::write(fd, bufferCursor, bufferEnd-bufferCursor);
+		if (writeResult == -1)
+		{
+			warn2e(__("unable to write to the file '%s'"), pipeName);
+			return;
+		}
+
+		bufferCursor += writeResult;
+	}
+	pipe.useAsReader(); // close the writer fd
+}
+
+class ScopedThread
+{
+ public:
+	ScopedThread(std::thread&& t)
+		: _t(std::move(t))
+	{}
+	~ScopedThread()
+	{
+		_t.join();
+	}
+ private:
+	std::thread _t;
+};
+
+static inline string getRedirectionSuffix(int fromFd, int toFd)
+{
+	return format2("%zu<&%zu", toFd, fromFd);
+}
+
+void WorkerBase::p_runCommandWithInput(Logger::Subsystem subsystem, Logger::Level level,
+		const string& command, const CommandInput& input)
+{
+	auto inputPipeName = format2("input stream for '%s'", command);
+	unique_ptr<Pipe> inputPipe(new Pipe(inputPipeName, true));
+
+	ScopedThread inputStreamThread(std::thread(writeBufferToFd,
+			std::ref(*inputPipe), std::cref(input.buffer), std::cref(inputPipeName)));
+
+	auto fdAmendedCommand = format2("bash -c '(%s) %s'", command, getRedirectionSuffix(inputPipe->getReaderFd(), input.fd));
+	p_invokeShellCommand(subsystem, level, fdAmendedCommand);
+}
+
 const string WorkerBase::partialDirectorySuffix = "/partial";
 
 }
