@@ -85,6 +85,8 @@ struct Changeset
 	vector< pair< download::Manager::DownloadEntity, string > > downloads;
 };
 
+class Dpkg;
+
 class PackagesWorker: public virtual WorkerBase
 {
 	std::set< string > __auto_installed_package_names;
@@ -96,10 +98,12 @@ class PackagesWorker: public virtual WorkerBase
 	map< string, pair< download::Manager::DownloadEntity, string > > __prepare_downloads();
 	vector< Changeset > __get_changesets(GraphAndAttributes&,
 			const map< string, pair< download::Manager::DownloadEntity, string > >&);
-	void __run_dpkg_command(const string&, const string&, const string&);
+	void __run_dpkg_command(const string&, const string&, const CommandInput&);
 	void __do_dpkg_pre_actions();
 	void __do_dpkg_post_actions();
-	string __generate_input_for_preinstall_v2_hooks(const vector< InnerActionGroup >&);
+	string p_generateInputForPreinstallV1Hooks(const vector<InnerActionGroup>&);
+	string p_generateInputForPreinstallV2OrV3Hooks(const vector<InnerActionGroup>&, bool);
+	CommandInput p_getCommandInputForPreinstallPackagesHook(const string&, const vector<InnerActionGroup>&);
 	void __do_dpkg_pre_packages_actions(const vector< InnerActionGroup >&);
 	void __clean_downloads(const Changeset& changeset);
 	void __do_downloads(const vector< pair< download::Manager::DownloadEntity, string > >&,
@@ -109,6 +113,7 @@ class PackagesWorker: public virtual WorkerBase
 	void __do_independent_auto_status_changes();
 	string __get_dpkg_action_command(const string&, const string&, const string&,
 			InnerAction::Type, const string&, const InnerActionGroup&, bool);
+	void p_processActionGroup(Dpkg&, const InnerActionGroup&);
  public:
 	PackagesWorker();
 
